@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,8 +44,6 @@
  * gst-launch-1.0 filesrc location=test.dv ! dvdemux name=demux ! queue ! audioconvert ! alsasink demux. ! queue ! dvdec ! xvimagesink
  * ]| This pipeline decodes and renders the raw DV stream to an audio and a videosink.
  * </refsect2>
- *
- * Last reviewed on 2006-02-27 (0.10.3)
  */
 
 /* DV output has two modes, normal and wide. The resolution is the same in both
@@ -143,10 +141,10 @@ static gboolean gst_dvdemux_sink_query (GstPad * pad, GstObject * parent,
 
 /* convert functions */
 static gboolean gst_dvdemux_sink_convert (GstDVDemux * demux,
-    GstFormat src_format, gint64 src_value, GstFormat * dest_format,
+    GstFormat src_format, gint64 src_value, GstFormat dest_format,
     gint64 * dest_value);
 static gboolean gst_dvdemux_src_convert (GstDVDemux * demux, GstPad * pad,
-    GstFormat src_format, gint64 src_value, GstFormat * dest_format,
+    GstFormat src_format, gint64 src_value, GstFormat dest_format,
     gint64 * dest_value);
 
 /* event functions */
@@ -260,7 +258,6 @@ gst_dvdemux_reset (GstDVDemux * dvdemux)
   dvdemux->frame_offset = 0;
   dvdemux->audio_offset = 0;
   dvdemux->video_offset = 0;
-  dvdemux->framecount = 0;
   g_atomic_int_set (&dvdemux->found_header, 0);
   dvdemux->frame_len = -1;
   dvdemux->need_segment = FALSE;
@@ -273,13 +270,40 @@ gst_dvdemux_reset (GstDVDemux * dvdemux)
   dvdemux->wide = FALSE;
   gst_segment_init (&dvdemux->byte_segment, GST_FORMAT_BYTES);
   gst_segment_init (&dvdemux->time_segment, GST_FORMAT_TIME);
+  dvdemux->have_group_id = FALSE;
+  dvdemux->group_id = G_MAXUINT;
+}
+
+static gboolean
+have_group_id (GstDVDemux * demux)
+{
+  GstEvent *event;
+
+  event = gst_pad_get_sticky_event (demux->sinkpad, GST_EVENT_STREAM_START, 0);
+  if (event) {
+    if (gst_event_parse_group_id (event, &demux->group_id))
+      demux->have_group_id = TRUE;
+    else
+      demux->have_group_id = FALSE;
+    gst_event_unref (event);
+  } else if (!demux->have_group_id) {
+    demux->have_group_id = TRUE;
+    demux->group_id = gst_util_group_id_next ();
+  }
+
+  return demux->have_group_id;
 }
 
 static GstPad *
-gst_dvdemux_add_pad (GstDVDemux * dvdemux, GstStaticPadTemplate * template)
+gst_dvdemux_add_pad (GstDVDemux * dvdemux, GstStaticPadTemplate * template,
+    GstCaps * caps)
 {
   gboolean no_more_pads;
   GstPad *pad;
+  GstEvent *event;
+  gchar *stream_id;
+  gchar rec_datetime[40];
+  GstDateTime *rec_dt;
 
   pad = gst_pad_new_from_static_template (template, template->name_template);
 
@@ -289,6 +313,21 @@ gst_dvdemux_add_pad (GstDVDemux * dvdemux, GstStaticPadTemplate * template)
       GST_DEBUG_FUNCPTR (gst_dvdemux_handle_src_event));
   gst_pad_use_fixed_caps (pad);
   gst_pad_set_active (pad, TRUE);
+
+  stream_id =
+      gst_pad_create_stream_id (pad,
+      GST_ELEMENT_CAST (dvdemux),
+      template == &video_src_temp ? "video" : "audio");
+  event = gst_event_new_stream_start (stream_id);
+  if (have_group_id (dvdemux))
+    gst_event_set_group_id (event, dvdemux->group_id);
+  gst_pad_push_event (pad, event);
+  g_free (stream_id);
+
+  gst_pad_set_caps (pad, caps);
+
+  gst_pad_push_event (pad, gst_event_new_segment (&dvdemux->time_segment));
+
   gst_element_add_pad (GST_ELEMENT (dvdemux), pad);
 
   no_more_pads =
@@ -298,14 +337,28 @@ gst_dvdemux_add_pad (GstDVDemux * dvdemux, GstStaticPadTemplate * template)
   if (no_more_pads)
     gst_element_no_more_pads (GST_ELEMENT (dvdemux));
 
-  gst_pad_push_event (pad, gst_event_new_segment (&dvdemux->time_segment));
-
   if (no_more_pads) {
     GstTagList *tags;
 
     tags = gst_tag_list_new (GST_TAG_CONTAINER_FORMAT, "DV", NULL);
     gst_tag_list_set_scope (tags, GST_TAG_SCOPE_GLOBAL);
-    gst_pad_push_event (pad, gst_event_new_tag (tags));
+
+    if (dv_get_recording_datetime (dvdemux->decoder, rec_datetime)) {
+      rec_dt = gst_date_time_new_from_iso8601_string (rec_datetime);
+      if (rec_dt) {
+        gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_DATE_TIME,
+            rec_dt, NULL);
+        gst_date_time_unref (rec_dt);
+      }
+    }
+
+    if (dvdemux->videosrcpad)
+      gst_pad_push_event (dvdemux->videosrcpad,
+          gst_event_new_tag (gst_tag_list_ref (tags)));
+    if (dvdemux->audiosrcpad)
+      gst_pad_push_event (dvdemux->audiosrcpad,
+          gst_event_new_tag (gst_tag_list_ref (tags)));
+    gst_tag_list_unref (tags);
   }
 
   return pad;
@@ -326,12 +379,12 @@ gst_dvdemux_remove_pads (GstDVDemux * dvdemux)
 
 static gboolean
 gst_dvdemux_src_convert (GstDVDemux * dvdemux, GstPad * pad,
-    GstFormat src_format, gint64 src_value, GstFormat * dest_format,
+    GstFormat src_format, gint64 src_value, GstFormat dest_format,
     gint64 * dest_value)
 {
   gboolean res = TRUE;
 
-  if (*dest_format == src_format || src_value == -1) {
+  if (dest_format == src_format || src_value == -1) {
     *dest_value = src_value;
     goto done;
   }
@@ -344,11 +397,11 @@ gst_dvdemux_src_convert (GstDVDemux * dvdemux, GstPad * pad,
 
   GST_INFO_OBJECT (pad,
       "src_value:%" G_GINT64_FORMAT ", src_format:%d, dest_format:%d",
-      src_value, src_format, *dest_format);
+      src_value, src_format, dest_format);
 
   switch (src_format) {
     case GST_FORMAT_BYTES:
-      switch (*dest_format) {
+      switch (dest_format) {
         case GST_FORMAT_DEFAULT:
           if (pad == dvdemux->videosrcpad)
             *dest_value = src_value / dvdemux->frame_len;
@@ -356,7 +409,6 @@ gst_dvdemux_src_convert (GstDVDemux * dvdemux, GstPad * pad,
             *dest_value = src_value / (2 * dvdemux->channels);
           break;
         case GST_FORMAT_TIME:
-          *dest_format = GST_FORMAT_TIME;
           if (pad == dvdemux->videosrcpad)
             *dest_value = gst_util_uint64_scale (src_value,
                 GST_SECOND * dvdemux->framerate_denominator,
@@ -370,7 +422,7 @@ gst_dvdemux_src_convert (GstDVDemux * dvdemux, GstPad * pad,
       }
       break;
     case GST_FORMAT_TIME:
-      switch (*dest_format) {
+      switch (dest_format) {
         case GST_FORMAT_BYTES:
           if (pad == dvdemux->videosrcpad)
             *dest_value = gst_util_uint64_scale (src_value,
@@ -398,7 +450,7 @@ gst_dvdemux_src_convert (GstDVDemux * dvdemux, GstPad * pad,
       }
       break;
     case GST_FORMAT_DEFAULT:
-      switch (*dest_format) {
+      switch (dest_format) {
         case GST_FORMAT_TIME:
           if (pad == dvdemux->videosrcpad) {
             *dest_value = gst_util_uint64_scale (src_value,
@@ -430,7 +482,7 @@ gst_dvdemux_src_convert (GstDVDemux * dvdemux, GstPad * pad,
 done:
   GST_INFO_OBJECT (pad,
       "Result : dest_format:%d, dest_value:%" G_GINT64_FORMAT ", res:%d",
-      *dest_format, *dest_value, res);
+      dest_format, *dest_value, res);
   return res;
 
   /* ERRORS */
@@ -443,19 +495,16 @@ error:
 
 static gboolean
 gst_dvdemux_sink_convert (GstDVDemux * dvdemux, GstFormat src_format,
-    gint64 src_value, GstFormat * dest_format, gint64 * dest_value)
+    gint64 src_value, GstFormat dest_format, gint64 * dest_value)
 {
   gboolean res = TRUE;
 
-  GST_DEBUG_OBJECT (dvdemux, "%d -> %d", src_format, *dest_format);
+  GST_DEBUG_OBJECT (dvdemux, "%d -> %d", src_format, dest_format);
   GST_INFO_OBJECT (dvdemux,
       "src_value:%" G_GINT64_FORMAT ", src_format:%d, dest_format:%d",
-      src_value, src_format, *dest_format);
+      src_value, src_format, dest_format);
 
-  if (*dest_format == GST_FORMAT_DEFAULT)
-    *dest_format = GST_FORMAT_TIME;
-
-  if (*dest_format == src_format || src_value == -1) {
+  if (dest_format == src_format || src_value == -1) {
     *dest_value = src_value;
     goto done;
   }
@@ -465,7 +514,7 @@ gst_dvdemux_sink_convert (GstDVDemux * dvdemux, GstFormat src_format,
 
   switch (src_format) {
     case GST_FORMAT_BYTES:
-      switch (*dest_format) {
+      switch (dest_format) {
         case GST_FORMAT_TIME:
         {
           guint64 frame;
@@ -484,7 +533,7 @@ gst_dvdemux_sink_convert (GstDVDemux * dvdemux, GstFormat src_format,
       }
       break;
     case GST_FORMAT_TIME:
-      switch (*dest_format) {
+      switch (dest_format) {
         case GST_FORMAT_BYTES:
         {
           guint64 frame;
@@ -507,7 +556,7 @@ gst_dvdemux_sink_convert (GstDVDemux * dvdemux, GstFormat src_format,
   }
   GST_INFO_OBJECT (dvdemux,
       "Result : dest_format:%d, dest_value:%" G_GINT64_FORMAT ", res:%d",
-      *dest_format, *dest_value, res);
+      dest_format, *dest_value, res);
 
 done:
   return res;
@@ -539,7 +588,7 @@ gst_dvdemux_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       /* bring the position to the requested format. */
       if (!(res = gst_dvdemux_src_convert (dvdemux, pad,
                   GST_FORMAT_TIME, dvdemux->time_segment.position,
-                  &format, &cur)))
+                  format, &cur)))
         goto error;
       gst_query_set_position (query, format, cur);
       break;
@@ -547,30 +596,29 @@ gst_dvdemux_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
     case GST_QUERY_DURATION:
     {
       GstFormat format;
-      GstFormat format2;
       gint64 end;
+      GstQuery *pquery;
 
       /* First ask the peer in the original format */
       if (!gst_pad_peer_query (dvdemux->sinkpad, query)) {
         /* get target format */
         gst_query_parse_duration (query, &format, NULL);
 
-        /* change query to bytes to perform on peer */
-        gst_query_set_duration (query, GST_FORMAT_BYTES, -1);
-
         /* Now ask the peer in BYTES format and try to convert */
-        if (!gst_pad_peer_query (dvdemux->sinkpad, query)) {
+        pquery = gst_query_new_duration (GST_FORMAT_BYTES);
+        if (!gst_pad_peer_query (dvdemux->sinkpad, pquery)) {
+          gst_query_unref (pquery);
           goto error;
         }
 
         /* get peer total length */
-        gst_query_parse_duration (query, NULL, &end);
+        gst_query_parse_duration (pquery, NULL, &end);
+        gst_query_unref (pquery);
 
         /* convert end to requested format */
         if (end != -1) {
-          format2 = format;
           if (!(res = gst_dvdemux_sink_convert (dvdemux,
-                      GST_FORMAT_BYTES, end, &format2, &end))) {
+                      GST_FORMAT_BYTES, end, format, &end))) {
             goto error;
           }
           gst_query_set_duration (query, format, end);
@@ -586,7 +634,7 @@ gst_dvdemux_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       gst_query_parse_convert (query, &src_fmt, &src_val, &dest_fmt, &dest_val);
       if (!(res =
               gst_dvdemux_src_convert (dvdemux, pad, src_fmt, src_val,
-                  &dest_fmt, &dest_val)))
+                  dest_fmt, &dest_val)))
         goto error;
       gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
       break;
@@ -622,7 +670,7 @@ gst_dvdemux_sink_query (GstPad * pad, GstObject * parent, GstQuery * query)
 
       gst_query_parse_convert (query, &src_fmt, &src_val, &dest_fmt, &dest_val);
       if (!(res =
-              gst_dvdemux_sink_convert (dvdemux, src_fmt, src_val, &dest_fmt,
+              gst_dvdemux_sink_convert (dvdemux, src_fmt, src_val, dest_fmt,
                   &dest_val)))
         goto error;
       gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
@@ -656,8 +704,10 @@ gst_dvdemux_push_event (GstDVDemux * dvdemux, GstEvent * event)
 
   if (dvdemux->audiosrcpad)
     res |= gst_pad_push_event (dvdemux->audiosrcpad, event);
-  else
+  else {
     gst_event_unref (event);
+    res = TRUE;
+  }
 
   return res;
 }
@@ -738,7 +788,7 @@ gst_dvdemux_convert_src_pair (GstDVDemux * dvdemux, GstPad * pad,
   GST_INFO ("starting conversion of start");
   /* bring the format to time on srcpad. */
   if (!(res = gst_dvdemux_src_convert (dvdemux, pad,
-              src_format, src_start, &dst_format, dst_start))) {
+              src_format, src_start, dst_format, dst_start))) {
     goto done;
   }
   GST_INFO ("Finished conversion of start: %" G_GINT64_FORMAT, *dst_start);
@@ -746,7 +796,7 @@ gst_dvdemux_convert_src_pair (GstDVDemux * dvdemux, GstPad * pad,
   GST_INFO ("starting conversion of stop");
   /* bring the format to time on srcpad. */
   if (!(res = gst_dvdemux_src_convert (dvdemux, pad,
-              src_format, src_stop, &dst_format, dst_stop))) {
+              src_format, src_stop, dst_format, dst_stop))) {
     /* could not convert seek format to time offset */
     goto done;
   }
@@ -766,7 +816,7 @@ gst_dvdemux_convert_sink_pair (GstDVDemux * dvdemux,
   GST_INFO ("starting conversion of start");
   /* bring the format to time on srcpad. */
   if (!(res = gst_dvdemux_sink_convert (dvdemux,
-              src_format, src_start, &dst_format, dst_start))) {
+              src_format, src_start, dst_format, dst_start))) {
     goto done;
   }
   GST_INFO ("Finished conversion of start: %" G_GINT64_FORMAT, *dst_start);
@@ -774,7 +824,7 @@ gst_dvdemux_convert_sink_pair (GstDVDemux * dvdemux,
   GST_INFO ("starting conversion of stop");
   /* bring the format to time on srcpad. */
   if (!(res = gst_dvdemux_sink_convert (dvdemux,
-              src_format, src_stop, &dst_format, dst_stop))) {
+              src_format, src_stop, dst_format, dst_stop))) {
     /* could not convert seek format to time offset */
     goto done;
   }
@@ -885,34 +935,34 @@ gst_dvdemux_do_seek (GstDVDemux * demux, GstSegment * segment)
   format = GST_FORMAT_BYTES;
   res = gst_dvdemux_sink_convert (demux,
       segment->format, segment->position,
-      &format, (gint64 *) & demux->byte_segment.position);
+      format, (gint64 *) & demux->byte_segment.position);
   if (!res)
     goto done;
 
   /* update byte segment start */
   gst_dvdemux_sink_convert (demux,
-      segment->format, segment->start, &format,
+      segment->format, segment->start, format,
       (gint64 *) & demux->byte_segment.start);
 
   /* update byte segment stop */
   gst_dvdemux_sink_convert (demux,
-      segment->format, segment->stop, &format,
+      segment->format, segment->stop, format,
       (gint64 *) & demux->byte_segment.stop);
 
   /* update byte segment time */
   gst_dvdemux_sink_convert (demux,
-      segment->format, segment->time, &format,
+      segment->format, segment->time, format,
       (gint64 *) & demux->byte_segment.time);
 
   /* calculate current frame number */
   format = GST_FORMAT_DEFAULT;
   gst_dvdemux_src_convert (demux, demux->videosrcpad,
-      segment->format, segment->start, &format, &demux->video_offset);
+      segment->format, segment->start, format, &demux->video_offset);
 
   /* calculate current audio number */
   format = GST_FORMAT_DEFAULT;
   gst_dvdemux_src_convert (demux, demux->audiosrcpad,
-      segment->format, segment->start, &format, &demux->audio_offset);
+      segment->format, segment->start, format, &demux->audio_offset);
 
   /* every DV frame corresponts with one video frame */
   demux->frame_offset = demux->video_offset;
@@ -1146,25 +1196,15 @@ gst_dvdemux_demux_audio (GstDVDemux * dvdemux, GstBuffer * buffer,
     GstBuffer *outbuf;
     gint frequency, channels;
 
-    if (G_UNLIKELY (dvdemux->audiosrcpad == NULL))
-      dvdemux->audiosrcpad = gst_dvdemux_add_pad (dvdemux, &audio_src_temp);
-
     /* get initial format or check if format changed */
     frequency = dv_get_frequency (dvdemux->decoder);
     channels = dv_get_num_channels (dvdemux->decoder);
 
-    if (G_UNLIKELY ((frequency != dvdemux->frequency)
+    if (G_UNLIKELY ((dvdemux->audiosrcpad == NULL)
+            || (frequency != dvdemux->frequency)
             || (channels != dvdemux->channels))) {
       GstCaps *caps;
       GstAudioInfo info;
-      gchar *stream_id;
-
-      stream_id =
-          gst_pad_create_stream_id (dvdemux->audiosrcpad,
-          GST_ELEMENT_CAST (dvdemux), "audio");
-      gst_pad_push_event (dvdemux->audiosrcpad,
-          gst_event_new_stream_start (stream_id));
-      g_free (stream_id);
 
       dvdemux->frequency = frequency;
       dvdemux->channels = channels;
@@ -1173,7 +1213,12 @@ gst_dvdemux_demux_audio (GstDVDemux * dvdemux, GstBuffer * buffer,
       gst_audio_info_set_format (&info, GST_AUDIO_FORMAT_S16LE,
           frequency, channels, NULL);
       caps = gst_audio_info_to_caps (&info);
-      gst_pad_set_caps (dvdemux->audiosrcpad, caps);
+      if (G_UNLIKELY (dvdemux->audiosrcpad == NULL)) {
+        dvdemux->audiosrcpad =
+            gst_dvdemux_add_pad (dvdemux, &audio_src_temp, caps);
+      } else {
+        gst_pad_set_caps (dvdemux->audiosrcpad, caps);
+      }
       gst_caps_unref (caps);
     }
 
@@ -1221,26 +1266,16 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
   gboolean wide;
   GstFlowReturn ret = GST_FLOW_OK;
 
-  if (G_UNLIKELY (dvdemux->videosrcpad == NULL))
-    dvdemux->videosrcpad = gst_dvdemux_add_pad (dvdemux, &video_src_temp);
-
   /* get params */
   /* framerate is already up-to-date */
   height = dvdemux->decoder->height;
   wide = dv_format_wide (dvdemux->decoder);
 
   /* see if anything changed */
-  if (G_UNLIKELY ((dvdemux->height != height) || dvdemux->wide != wide)) {
-    GstCaps *caps;
+  if (G_UNLIKELY ((dvdemux->videosrcpad == NULL) || (dvdemux->height != height)
+          || dvdemux->wide != wide)) {
     gint par_x, par_y;
-    gchar *stream_id;
-
-    stream_id =
-        gst_pad_create_stream_id (dvdemux->videosrcpad,
-        GST_ELEMENT_CAST (dvdemux), "video");
-    gst_pad_push_event (dvdemux->videosrcpad,
-        gst_event_new_stream_start (stream_id));
-    g_free (stream_id);
+    GstCaps *caps;
 
     dvdemux->height = height;
     dvdemux->wide = wide;
@@ -1270,7 +1305,13 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
         "framerate", GST_TYPE_FRACTION, dvdemux->framerate_numerator,
         dvdemux->framerate_denominator,
         "pixel-aspect-ratio", GST_TYPE_FRACTION, par_x, par_y, NULL);
-    gst_pad_set_caps (dvdemux->videosrcpad, caps);
+
+    if (G_UNLIKELY (dvdemux->videosrcpad == NULL)) {
+      dvdemux->videosrcpad =
+          gst_dvdemux_add_pad (dvdemux, &video_src_temp, caps);
+    } else {
+      gst_pad_set_caps (dvdemux->videosrcpad, caps);
+    }
     gst_caps_unref (caps);
   }
 
@@ -1404,7 +1445,7 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
     format = GST_FORMAT_DEFAULT;
     if (!(gst_dvdemux_src_convert (dvdemux, dvdemux->videosrcpad,
                 GST_FORMAT_TIME, dvdemux->time_segment.start,
-                &format, &dvdemux->frame_offset)))
+                format, &dvdemux->frame_offset)))
       goto segment_error;
 
     GST_DEBUG_OBJECT (dvdemux, "sending segment start: %" GST_TIME_FORMAT
@@ -1666,7 +1707,6 @@ gst_dvdemux_loop (GstPad * pad)
     }
   }
 
-
   if (G_UNLIKELY (dvdemux->pending_segment)) {
 
     /* now send the newsegment */
