@@ -111,6 +111,11 @@ gst_pngdec_init (GstPngDec * pngdec)
   pngdec->color_type = -1;
 
   pngdec->image_ready = FALSE;
+  pngdec->read_data = 0;
+
+  gst_video_decoder_set_use_default_pad_acceptcaps (GST_VIDEO_DECODER_CAST
+      (pngdec), TRUE);
+  GST_PAD_SET_ACCEPT_TEMPLATE (GST_VIDEO_DECODER_SINK_PAD (pngdec));
 }
 
 static void
@@ -129,7 +134,7 @@ static void
 user_info_callback (png_structp png_ptr, png_infop info)
 {
   GstPngDec *pngdec = NULL;
-  GstFlowReturn ret = GST_FLOW_OK;
+  GstFlowReturn ret;
 
   GST_LOG ("info ready");
 
@@ -155,13 +160,12 @@ static gboolean
 gst_pngdec_set_format (GstVideoDecoder * decoder, GstVideoCodecState * state)
 {
   GstPngDec *pngdec = (GstPngDec *) decoder;
-  GstVideoInfo *info = &state->info;
 
   if (pngdec->input_state)
     gst_video_codec_state_unref (pngdec->input_state);
   pngdec->input_state = gst_video_codec_state_ref (state);
 
-  if (GST_VIDEO_INFO_FPS_N (info) != 1 && GST_VIDEO_INFO_FPS_D (info) != 1)
+  if (decoder->input_segment.format == GST_FORMAT_TIME)
     gst_video_decoder_set_packetized (decoder, TRUE);
   else
     gst_video_decoder_set_packetized (decoder, FALSE);
@@ -179,15 +183,12 @@ user_endrow_callback (png_structp png_ptr, png_bytep new_row,
 
   pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
 
-  /* FIXME: implement interlaced pictures */
-
   /* If buffer_out doesn't exist, it means buffer_alloc failed, which 
    * will already have set the return code */
-  if (GST_IS_BUFFER (pngdec->current_frame->output_buffer)) {
+  if (new_row && GST_IS_BUFFER (pngdec->current_frame->output_buffer)) {
     GstVideoFrame frame;
     GstBuffer *buffer = pngdec->current_frame->output_buffer;
     size_t offset;
-    gint width;
     guint8 *data;
 
     if (!gst_video_frame_map (&frame, &pngdec->output_state->info, buffer,
@@ -198,13 +199,14 @@ user_endrow_callback (png_structp png_ptr, png_bytep new_row,
 
     data = GST_VIDEO_FRAME_COMP_DATA (&frame, 0);
     offset = row_num * GST_VIDEO_FRAME_COMP_STRIDE (&frame, 0);
-    GST_LOG ("got row %u, copying in buffer %p at offset %" G_GSIZE_FORMAT,
-        (guint) row_num, pngdec->current_frame->output_buffer, offset);
-    width = GST_ROUND_UP_4 (png_get_rowbytes (pngdec->png, pngdec->info));
-    memcpy (data + offset, new_row, width);
+    GST_LOG ("got row %u at pass %d, copying in buffer %p at offset %"
+        G_GSIZE_FORMAT, (guint) row_num, pass,
+        pngdec->current_frame->output_buffer, offset);
+    png_progressive_combine_row (pngdec->png, data + offset, new_row);
     gst_video_frame_unmap (&frame);
     pngdec->ret = GST_FLOW_OK;
-  }
+  } else
+    pngdec->ret = GST_FLOW_OK;
 }
 
 static void
@@ -286,6 +288,8 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
     png_set_palette_to_rgb (pngdec->png);
   }
 
+  png_set_interlace_handling (pngdec->png);
+
   /* Update the info structure */
   png_read_update_info (pngdec->png, pngdec->info);
 
@@ -412,6 +416,9 @@ gst_pngdec_parse (GstVideoDecoder * decoder, GstVideoCodecFrame * frame,
   gconstpointer data;
   guint64 signature;
   gsize size;
+  GstPngDec *pngdec = (GstPngDec *) decoder;
+
+  GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (frame);
 
   /* FIXME : The overhead of using scan_uint32 is massive */
 
@@ -424,39 +431,42 @@ gst_pngdec_parse (GstVideoDecoder * decoder, GstVideoCodecFrame * frame,
   data = gst_adapter_map (adapter, size);
   gst_byte_reader_init (&reader, data, size);
 
-  if (!gst_byte_reader_peek_uint64_be (&reader, &signature))
-    goto need_more_data;
+  if (pngdec->read_data == 0) {
+    if (!gst_byte_reader_peek_uint64_be (&reader, &signature))
+      goto need_more_data;
 
-  if (signature != PNG_SIGNATURE) {
-    for (;;) {
-      guint offset;
+    if (signature != PNG_SIGNATURE) {
+      for (;;) {
+        guint offset;
 
-      offset = gst_byte_reader_masked_scan_uint32 (&reader, 0xffffffff,
-          0x89504E47, 0, gst_byte_reader_get_remaining (&reader));
+        offset = gst_byte_reader_masked_scan_uint32 (&reader, 0xffffffff,
+            0x89504E47, 0, gst_byte_reader_get_remaining (&reader));
 
-      if (offset == -1) {
-        gst_adapter_flush (adapter,
-            gst_byte_reader_get_remaining (&reader) - 4);
-        goto need_more_data;
+        if (offset == -1) {
+          gst_adapter_flush (adapter,
+              gst_byte_reader_get_remaining (&reader) - 4);
+          goto need_more_data;
+        }
+
+        if (!gst_byte_reader_skip (&reader, offset))
+          goto need_more_data;
+
+        if (!gst_byte_reader_peek_uint64_be (&reader, &signature))
+          goto need_more_data;
+
+        if (signature == PNG_SIGNATURE) {
+          /* We're skipping, go out, we'll be back */
+          gst_adapter_flush (adapter, gst_byte_reader_get_pos (&reader));
+          goto need_more_data;
+        }
+        if (!gst_byte_reader_skip (&reader, 4))
+          goto need_more_data;
       }
-
-      if (!gst_byte_reader_skip (&reader, offset))
-        goto need_more_data;
-
-      if (!gst_byte_reader_peek_uint64_be (&reader, &signature))
-        goto need_more_data;
-
-      if (signature == PNG_SIGNATURE) {
-        /* We're skipping, go out, we'll be back */
-        gst_adapter_flush (adapter, gst_byte_reader_get_pos (&reader));
-        goto need_more_data;
-      }
-      if (!gst_byte_reader_skip (&reader, 4))
-        goto need_more_data;
     }
+    pngdec->read_data = 8;
   }
 
-  if (!gst_byte_reader_skip (&reader, 8))
+  if (!gst_byte_reader_skip (&reader, pngdec->read_data))
     goto need_more_data;
 
   for (;;) {
@@ -476,8 +486,10 @@ gst_pngdec_parse (GstVideoDecoder * decoder, GstVideoCodecFrame * frame,
       toadd = gst_byte_reader_get_pos (&reader);
       GST_DEBUG_OBJECT (decoder, "Have complete frame of size %" G_GSIZE_FORMAT,
           toadd);
+      pngdec->read_data = 0;
       goto have_full_frame;
-    }
+    } else
+      pngdec->read_data += length + 12;
   }
 
   g_assert_not_reached ();
@@ -590,6 +602,7 @@ gst_pngdec_libpng_clear (GstPngDec * pngdec)
   }
 
   pngdec->color_type = -1;
+  pngdec->read_data = 0;
 }
 
 static gboolean
