@@ -94,6 +94,7 @@ enum
 #define SUPPORTED_CAPS \
 GST_STATIC_CAPS ( \
     GST_AUDIO_CAPS_MAKE (GST_AUDIO_NE (F32)) "; " \
+    GST_AUDIO_CAPS_MAKE (GST_AUDIO_NE (F64)) "; " \
     GST_AUDIO_CAPS_MAKE (GST_AUDIO_NE (S16)) \
 )
 
@@ -113,40 +114,44 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
 G_DEFINE_TYPE_WITH_CODE (GstScaletempo, gst_scaletempo,
     GST_TYPE_BASE_TRANSFORM, DEBUG_INIT (0));
 
-static guint
-best_overlap_offset_float (GstScaletempo * st)
-{
-  gfloat *pw, *po, *ppc, *search_start;
-  gfloat best_corr = G_MININT;
-  guint best_off = 0;
-  gint i, off;
-
-  pw = st->table_window;
-  po = st->buf_overlap;
-  po += st->samples_per_frame;
-  ppc = st->buf_pre_corr;
-  for (i = st->samples_per_frame; i < st->samples_overlap; i++) {
-    *ppc++ = *pw++ * *po++;
-  }
-
-  search_start = (gfloat *) st->buf_queue + st->samples_per_frame;
-  for (off = 0; off < st->frames_search; off++) {
-    gfloat corr = 0;
-    gfloat *ps = search_start;
-    ppc = st->buf_pre_corr;
-    for (i = st->samples_per_frame; i < st->samples_overlap; i++) {
-      corr += *ppc++ * *ps++;
-    }
-    if (corr > best_corr) {
-      best_corr = corr;
-      best_off = off;
-    }
-    search_start += st->samples_per_frame;
-  }
-
-  return best_off * st->bytes_per_frame;
+#define CREATE_BEST_OVERLAP_OFFSET_FLOAT_FUNC(type) \
+static guint \
+best_overlap_offset_##type (GstScaletempo * st) \
+{ \
+  g##type *pw, *po, *ppc, *search_start; \
+  g##type best_corr = G_MININT; \
+  guint best_off = 0; \
+  gint i, off; \
+  \
+  pw = st->table_window; \
+  po = st->buf_overlap; \
+  po += st->samples_per_frame; \
+  ppc = st->buf_pre_corr; \
+  for (i = st->samples_per_frame; i < st->samples_overlap; i++) { \
+    *ppc++ = *pw++ * *po++; \
+  } \
+  \
+  search_start = (g##type *) st->buf_queue + st->samples_per_frame; \
+  for (off = 0; off < st->frames_search; off++) { \
+    g##type corr = 0; \
+    g##type *ps = search_start; \
+    ppc = st->buf_pre_corr; \
+    for (i = st->samples_per_frame; i < st->samples_overlap; i++) { \
+      corr += *ppc++ * *ps++; \
+    } \
+    if (corr > best_corr) { \
+      best_corr = corr; \
+      best_off = off; \
+    } \
+    search_start += st->samples_per_frame; \
+  } \
+  \
+  return best_off * st->bytes_per_frame; \
 }
 
+CREATE_BEST_OVERLAP_OFFSET_FLOAT_FUNC (float);
+CREATE_BEST_OVERLAP_OFFSET_FLOAT_FUNC (double);
+
 /* buffer padding for loop optimization: sizeof(gint32) * (loop_size - 1) */
 #define UNROLL_PADDING (4*3)
 static guint
@@ -192,20 +197,24 @@ best_overlap_offset_s16 (GstScaletempo * st)
   return best_off * st->bytes_per_frame;
 }
 
-static void
-output_overlap_float (GstScaletempo * st, gpointer buf_out, guint bytes_off)
-{
-  gfloat *pout = buf_out;
-  gfloat *pb = st->table_blend;
-  gfloat *po = st->buf_overlap;
-  gfloat *pin = (gfloat *) (st->buf_queue + bytes_off);
-  gint i;
-  for (i = 0; i < st->samples_overlap; i++) {
-    *pout++ = *po - *pb++ * (*po - *pin++);
-    po++;
-  }
+#define CREATE_OUTPUT_OVERLAP_FLOAT_FUNC(type) \
+static void \
+output_overlap_##type (GstScaletempo * st, gpointer buf_out, guint bytes_off) \
+{ \
+  g##type *pout = buf_out; \
+  g##type *pb = st->table_blend; \
+  g##type *po = st->buf_overlap; \
+  g##type *pin = (g##type *) (st->buf_queue + bytes_off); \
+  gint i; \
+  for (i = 0; i < st->samples_overlap; i++) { \
+    *pout++ = *po - *pb++ * (*po - *pin++); \
+    po++; \
+  } \
 }
 
+CREATE_OUTPUT_OVERLAP_FLOAT_FUNC (float);
+CREATE_OUTPUT_OVERLAP_FLOAT_FUNC (double);
+
 static void
 output_overlap_s16 (GstScaletempo * st, gpointer buf_out, guint bytes_off)
 {
@@ -283,12 +292,13 @@ reinit_buffers (GstScaletempo * st)
     st->bytes_standing = st->bytes_stride - st->bytes_overlap;
     st->samples_standing = st->bytes_standing / st->bytes_per_sample;
     st->buf_overlap = g_realloc (st->buf_overlap, st->bytes_overlap);
-    st->table_blend = g_realloc (st->table_blend, st->samples_overlap * 4);     /* sizeof (gint32|gfloat) */
+    st->table_blend =
+        g_realloc (st->table_blend, st->samples_overlap * st->bytes_per_sample);
     if (st->bytes_overlap > prev_overlap) {
       memset ((guint8 *) st->buf_overlap + prev_overlap, 0,
           st->bytes_overlap - prev_overlap);
     }
-    if (st->use_int) {
+    if (st->format == GST_AUDIO_FORMAT_S16) {
       gint32 *pb = st->table_blend;
       gint64 blend = 0;
       for (i = 0; i < frames_overlap; i++) {
@@ -299,7 +309,7 @@ reinit_buffers (GstScaletempo * st)
         blend += 65535;         /* 2^16 */
       }
       st->output_overlap = output_overlap_s16;
-    } else {
+    } else if (st->format == GST_AUDIO_FORMAT_F32) {
       gfloat *pb = st->table_blend;
       gfloat t = (gfloat) frames_overlap;
       for (i = 0; i < frames_overlap; i++) {
@@ -309,6 +319,16 @@ reinit_buffers (GstScaletempo * st)
         }
       }
       st->output_overlap = output_overlap_float;
+    } else {
+      gdouble *pb = st->table_blend;
+      gdouble t = (gdouble) frames_overlap;
+      for (i = 0; i < frames_overlap; i++) {
+        gdouble v = i / t;
+        for (j = 0; j < st->samples_per_frame; j++) {
+          *pb++ = v;
+        }
+      }
+      st->output_overlap = output_overlap_double;
     }
   }
 
@@ -318,11 +338,12 @@ reinit_buffers (GstScaletempo * st)
   if (st->frames_search < 1) {  /* if no search */
     st->best_overlap_offset = NULL;
   } else {
-    guint bytes_pre_corr = (st->samples_overlap - st->samples_per_frame) * 4;   /* sizeof (gint32|gfloat) */
+    guint bytes_pre_corr =
+        (st->samples_overlap - st->samples_per_frame) * st->bytes_per_sample;
     st->buf_pre_corr =
         g_realloc (st->buf_pre_corr, bytes_pre_corr + UNROLL_PADDING);
     st->table_window = g_realloc (st->table_window, bytes_pre_corr);
-    if (st->use_int) {
+    if (st->format == GST_AUDIO_FORMAT_S16) {
       gint64 t = frames_overlap;
       gint32 n = 8589934588LL / (t * t);        /* 4 * (2^31 - 1) / t^2 */
       gint32 *pw;
@@ -336,7 +357,7 @@ reinit_buffers (GstScaletempo * st)
         }
       }
       st->best_overlap_offset = best_overlap_offset_s16;
-    } else {
+    } else if (st->format == GST_AUDIO_FORMAT_F32) {
       gfloat *pw = st->table_window;
       for (i = 1; i < frames_overlap; i++) {
         gfloat v = i * (frames_overlap - i);
@@ -345,6 +366,15 @@ reinit_buffers (GstScaletempo * st)
         }
       }
       st->best_overlap_offset = best_overlap_offset_float;
+    } else {
+      gdouble *pw = st->table_window;
+      for (i = 1; i < frames_overlap; i++) {
+        gdouble v = i * (frames_overlap - i);
+        for (j = 0; j < st->samples_per_frame; j++) {
+          *pw++ = v;
+        }
+      }
+      st->best_overlap_offset = best_overlap_offset_double;
     }
   }
 
@@ -386,11 +416,52 @@ reinit_buffers (GstScaletempo * st)
       (gint) (st->bytes_standing / st->bytes_per_frame),
       (gint) (st->bytes_overlap / st->bytes_per_frame), st->frames_search,
       (gint) (st->bytes_queue_max / st->bytes_per_frame),
-      (st->use_int ? "s16" : "float"));
+      gst_audio_format_to_string (st->format));
 
   st->reinit_buffers = FALSE;
 }
 
+static GstBuffer *
+reverse_buffer (GstScaletempo * st, GstBuffer * inbuf)
+{
+  GstBuffer *outbuf;
+  GstMapInfo imap, omap;
+
+  gst_buffer_map (inbuf, &imap, GST_MAP_READ);
+  outbuf = gst_buffer_new_and_alloc (imap.size);
+  gst_buffer_map (outbuf, &omap, GST_MAP_WRITE);
+
+  if (st->format == GST_AUDIO_FORMAT_F64) {
+    const gint64 *ip = (const gint64 *) imap.data;
+    gint64 *op = (gint64 *) (omap.data + omap.size - 8 * st->samples_per_frame);
+    guint i, n = imap.size / (8 * st->samples_per_frame);
+    guint j, c = st->samples_per_frame;
+
+    for (i = 0; i < n; i++) {
+      for (j = 0; j < c; j++)
+        op[j] = ip[j];
+      op -= c;
+      ip += c;
+    }
+  } else {
+    const gint32 *ip = (const gint32 *) imap.data;
+    gint32 *op = (gint32 *) (omap.data + omap.size - 4 * st->samples_per_frame);
+    guint i, n = imap.size / (4 * st->samples_per_frame);
+    guint j, c = st->samples_per_frame;
+
+    for (i = 0; i < n; i++) {
+      for (j = 0; j < c; j++)
+        op[j] = ip[j];
+      op -= c;
+      ip += c;
+    }
+  }
+
+  gst_buffer_unmap (inbuf, &imap);
+  gst_buffer_unmap (outbuf, &omap);
+
+  return outbuf;
+}
 
 /* GstBaseTransform vmethod implementations */
 static GstFlowReturn
@@ -402,10 +473,16 @@ gst_scaletempo_transform (GstBaseTransform * trans,
   guint offset_in, bytes_out;
   GstMapInfo omap;
   GstClockTime timestamp;
+  GstBuffer *tmpbuf = NULL;
+
+  if (st->reverse)
+    tmpbuf = reverse_buffer (st, inbuf);
 
   gst_buffer_map (outbuf, &omap, GST_MAP_WRITE);
   pout = (gint8 *) omap.data;
-  offset_in = fill_queue (st, inbuf, 0);
+  bytes_out = omap.size;
+
+  offset_in = fill_queue (st, tmpbuf ? tmpbuf : inbuf, 0);
   bytes_out = 0;
   while (st->bytes_queued >= st->bytes_queue_max) {
     guint bytes_off = 0;
@@ -432,25 +509,53 @@ gst_scaletempo_transform (GstBaseTransform * trans,
     st->bytes_to_slide = frames_to_stride_whole * st->bytes_per_frame;
     st->frames_stride_error = frames_to_slide - frames_to_stride_whole;
 
-    offset_in += fill_queue (st, inbuf, offset_in);
+    offset_in += fill_queue (st, tmpbuf ? tmpbuf : inbuf, offset_in);
   }
-
   gst_buffer_unmap (outbuf, &omap);
 
-  timestamp = GST_BUFFER_TIMESTAMP (inbuf) - st->segment_start;
-  if (timestamp < st->latency)
-    timestamp = 0;
-  else
-    timestamp -= st->latency;
-  GST_BUFFER_TIMESTAMP (outbuf) = timestamp / st->scale + st->segment_start;
+  if (st->reverse) {
+    timestamp = st->in_segment.stop - GST_BUFFER_TIMESTAMP (inbuf);
+    if (timestamp < st->latency)
+      timestamp = 0;
+    else
+      timestamp -= st->latency;
+  } else {
+    timestamp = GST_BUFFER_TIMESTAMP (inbuf) - st->in_segment.start;
+    if (timestamp < st->latency)
+      timestamp = 0;
+    else
+      timestamp -= st->latency;
+  }
+  GST_BUFFER_TIMESTAMP (outbuf) = timestamp / st->scale + st->in_segment.start;
   GST_BUFFER_DURATION (outbuf) =
       gst_util_uint64_scale (bytes_out, GST_SECOND,
       st->bytes_per_frame * st->sample_rate);
   gst_buffer_set_size (outbuf, bytes_out);
 
+  if (tmpbuf)
+    gst_buffer_unref (tmpbuf);
+
   return GST_FLOW_OK;
 }
 
+static GstFlowReturn
+gst_scaletempo_submit_input_buffer (GstBaseTransform * trans,
+    gboolean is_discont, GstBuffer * input)
+{
+  GstScaletempo *scaletempo = GST_SCALETEMPO (trans);
+
+  if (scaletempo->in_segment.format == GST_FORMAT_TIME) {
+    input =
+        gst_audio_buffer_clip (input, &scaletempo->in_segment,
+        scaletempo->sample_rate, scaletempo->bytes_per_frame);
+    if (!input)
+      return GST_FLOW_OK;
+  }
+
+  return GST_BASE_TRANSFORM_CLASS (parent_class)->submit_input_buffer (trans,
+      is_discont, input);
+}
+
 static gboolean
 gst_scaletempo_transform_size (GstBaseTransform * trans,
     GstPadDirection direction,
@@ -482,21 +587,25 @@ gst_scaletempo_transform_size (GstBaseTransform * trans,
 static gboolean
 gst_scaletempo_sink_event (GstBaseTransform * trans, GstEvent * event)
 {
+  GstScaletempo *scaletempo = GST_SCALETEMPO (trans);
+
   if (GST_EVENT_TYPE (event) == GST_EVENT_SEGMENT) {
-    GstScaletempo *scaletempo = GST_SCALETEMPO (trans);
     GstSegment segment;
 
     gst_event_copy_segment (event, &segment);
 
-    if (scaletempo->scale != segment.rate) {
-      if (ABS (segment.rate - 1.0) < 1e-10) {
+    if (segment.format != GST_FORMAT_TIME
+        || scaletempo->scale != ABS (segment.rate)
+        || ! !scaletempo->reverse != ! !(segment.rate < 0.0)) {
+      if (segment.format != GST_FORMAT_TIME || ABS (segment.rate - 1.0) < 1e-10) {
         scaletempo->scale = 1.0;
         gst_base_transform_set_passthrough (GST_BASE_TRANSFORM (scaletempo),
             TRUE);
       } else {
         gst_base_transform_set_passthrough (GST_BASE_TRANSFORM (scaletempo),
             FALSE);
-        scaletempo->scale = segment.rate;
+        scaletempo->scale = ABS (segment.rate);
+        scaletempo->reverse = segment.rate < 0.0;
         scaletempo->bytes_stride_scaled =
             scaletempo->bytes_stride * scaletempo->scale;
         scaletempo->frames_stride_scaled =
@@ -509,22 +618,36 @@ gst_scaletempo_sink_event (GstBaseTransform * trans, GstEvent * event)
       }
     }
 
-    if (scaletempo->scale != 1.0) {
-      scaletempo->segment_start = segment.start;
-      segment.applied_rate = scaletempo->scale;
+    scaletempo->in_segment = segment;
+    scaletempo->out_segment = segment;
+
+    if (scaletempo->scale != 1.0 || scaletempo->reverse) {
+      guint32 seqnum;
+
+      segment.applied_rate = segment.rate;
       segment.rate = 1.0;
-      gst_event_unref (event);
 
       if (segment.stop != -1) {
-        segment.stop = (segment.stop - segment.start) / segment.applied_rate +
+        segment.stop =
+            (segment.stop - segment.start) / ABS (segment.applied_rate) +
             segment.start;
       }
 
+      scaletempo->out_segment = segment;
+
+      seqnum = gst_event_get_seqnum (event);
+      gst_event_unref (event);
+
       event = gst_event_new_segment (&segment);
-      gst_pad_push_event (GST_BASE_TRANSFORM_SRC_PAD (trans), event);
-      return TRUE;
+      gst_event_set_seqnum (event, seqnum);
+
+      return gst_pad_push_event (GST_BASE_TRANSFORM_SRC_PAD (trans), event);
     }
+  } else if (GST_EVENT_TYPE (event) == GST_EVENT_FLUSH_STOP) {
+    gst_segment_init (&scaletempo->in_segment, GST_FORMAT_UNDEFINED);
+    gst_segment_init (&scaletempo->out_segment, GST_FORMAT_UNDEFINED);
   }
+
   return GST_BASE_TRANSFORM_CLASS (parent_class)->sink_event (trans, event);
 }
 
@@ -535,8 +658,8 @@ gst_scaletempo_set_caps (GstBaseTransform * trans,
   GstScaletempo *scaletempo = GST_SCALETEMPO (trans);
 
   gint width, bps, nch, rate;
-  gboolean use_int;
   GstAudioInfo info;
+  GstAudioFormat format;
 
   if (!gst_audio_info_from_caps (&info, incaps))
     return FALSE;
@@ -544,7 +667,7 @@ gst_scaletempo_set_caps (GstBaseTransform * trans,
   nch = GST_AUDIO_INFO_CHANNELS (&info);
   rate = GST_AUDIO_INFO_RATE (&info);
   width = GST_AUDIO_INFO_WIDTH (&info);
-  use_int = GST_AUDIO_INFO_IS_INTEGER (&info);
+  format = GST_AUDIO_INFO_FORMAT (&info);
 
   bps = width / 8;
 
@@ -552,19 +675,48 @@ gst_scaletempo_set_caps (GstBaseTransform * trans,
 
   if (rate != scaletempo->sample_rate
       || nch != scaletempo->samples_per_frame
-      || bps != scaletempo->bytes_per_sample
-      || use_int != scaletempo->use_int) {
+      || bps != scaletempo->bytes_per_sample || format != scaletempo->format) {
     scaletempo->sample_rate = rate;
     scaletempo->samples_per_frame = nch;
     scaletempo->bytes_per_sample = bps;
     scaletempo->bytes_per_frame = nch * bps;
-    scaletempo->use_int = use_int;
+    scaletempo->format = format;
     scaletempo->reinit_buffers = TRUE;
   }
 
   return TRUE;
 }
 
+static gboolean
+gst_scaletempo_start (GstBaseTransform * trans)
+{
+  GstScaletempo *scaletempo = GST_SCALETEMPO (trans);
+
+  gst_segment_init (&scaletempo->in_segment, GST_FORMAT_UNDEFINED);
+  gst_segment_init (&scaletempo->out_segment, GST_FORMAT_UNDEFINED);
+
+  return TRUE;
+}
+
+static gboolean
+gst_scaletempo_stop (GstBaseTransform * trans)
+{
+  GstScaletempo *scaletempo = GST_SCALETEMPO (trans);
+
+  g_free (scaletempo->buf_queue);
+  scaletempo->buf_queue = NULL;
+  g_free (scaletempo->buf_overlap);
+  scaletempo->buf_overlap = NULL;
+  g_free (scaletempo->table_blend);
+  scaletempo->table_blend = NULL;
+  g_free (scaletempo->buf_pre_corr);
+  scaletempo->buf_pre_corr = NULL;
+  g_free (scaletempo->table_window);
+  scaletempo->table_window = NULL;
+
+  return TRUE;
+}
+
 static gboolean
 gst_scaletempo_query (GstBaseTransform * trans, GstPadDirection direction,
     GstQuery * query)
@@ -575,10 +727,9 @@ gst_scaletempo_query (GstBaseTransform * trans, GstPadDirection direction,
     switch (GST_QUERY_TYPE (query)) {
       case GST_QUERY_LATENCY:{
         GstPad *peer;
-        gboolean res;
 
         if ((peer = gst_pad_get_peer (GST_BASE_TRANSFORM_SINK_PAD (trans)))) {
-          if ((res = gst_pad_query (peer, query))) {
+          if ((gst_pad_query (peer, query))) {
             GstClockTime min, max;
             gboolean live;
 
@@ -604,12 +755,10 @@ gst_scaletempo_query (GstBaseTransform * trans, GstPadDirection direction,
         }
 
         return TRUE;
-        break;
       }
       default:{
         return GST_BASE_TRANSFORM_CLASS (parent_class)->query (trans, direction,
             query);
-        break;
       }
     }
   } else {
@@ -726,6 +875,10 @@ gst_scaletempo_class_init (GstScaletempoClass * klass)
       GST_DEBUG_FUNCPTR (gst_scaletempo_transform_size);
   basetransform_class->transform = GST_DEBUG_FUNCPTR (gst_scaletempo_transform);
   basetransform_class->query = GST_DEBUG_FUNCPTR (gst_scaletempo_query);
+  basetransform_class->start = GST_DEBUG_FUNCPTR (gst_scaletempo_start);
+  basetransform_class->stop = GST_DEBUG_FUNCPTR (gst_scaletempo_stop);
+  basetransform_class->submit_input_buffer =
+      GST_DEBUG_FUNCPTR (gst_scaletempo_submit_input_buffer);
 }
 
 static void
@@ -743,5 +896,6 @@ gst_scaletempo_init (GstScaletempo * scaletempo)
   scaletempo->bytes_stride = 0;
   scaletempo->bytes_queued = 0;
   scaletempo->bytes_to_slide = 0;
-  scaletempo->segment_start = 0;
+  gst_segment_init (&scaletempo->in_segment, GST_FORMAT_UNDEFINED);
+  gst_segment_init (&scaletempo->out_segment, GST_FORMAT_UNDEFINED);
 }
