@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __GST_AVI_DEMUX_H__
@@ -27,6 +27,7 @@
 #include "gst/riff/riff-ids.h"
 #include "gst/riff/riff-read.h"
 #include <gst/base/gstadapter.h>
+#include <gst/base/gstflowcombiner.h>
 
 G_BEGIN_DECLS
 
@@ -73,6 +74,7 @@ typedef struct {
     gpointer     data;
   } strf;
   GstBuffer     *extradata, *initdata;
+  GstBuffer     *rgb8_palette;
   gchar         *name;
 
   /* the start/step/stop entries */
@@ -89,7 +91,6 @@ typedef struct {
   guint64        current_offset;
   guint64        current_offset_end;
 
-  GstFlowReturn  last_flow;
   gboolean       discont;
 
   /* stream length */
@@ -118,6 +119,7 @@ typedef struct {
   GstTagList	*taglist;
 
   gint           index_id;
+  gboolean is_raw;
 } GstAviStream;
 
 typedef enum {
@@ -158,10 +160,16 @@ typedef struct _GstAviDemux {
   guint          num_streams;
   guint          num_v_streams;
   guint          num_a_streams;
-  guint          num_t_streams;  /* subtitle text streams */
+  guint          num_t_streams;   /* subtitle text streams */
+  guint          num_sp_streams;  /* subpicture streams */
 
   guint          main_stream; /* used for seeking */
 
+  GstFlowCombiner *flowcombiner;
+
+  gboolean       have_group_id;
+  guint          group_id;
+
   /* for streaming mode */
   gboolean       streaming;
   gboolean       have_eos;
