@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 /**
  * SECTION:element-taginject
@@ -27,10 +27,10 @@
  * <refsect2>
  * <title>Example launch lines</title>
  * |[
- * gst-launch audiotestsrc num-buffers=100 ! taginject tags="title=testsrc,artist=gstreamer" ! vorbisenc ! oggmux ! filesink location=test.ogg
+ * gst-launch-1.0 audiotestsrc num-buffers=100 ! taginject tags="title=testsrc,artist=gstreamer" ! vorbisenc ! oggmux ! filesink location=test.ogg
  * ]| set title and artist
  * |[
- * gst-launch audiotestsrc num-buffers=100 ! taginject tags="keywords=\{\"testone\",\"audio\"\},title=\"audio testtone\"" ! vorbisenc ! oggmux ! filesink location=test.ogg
+ * gst-launch-1.0 audiotestsrc num-buffers=100 ! taginject tags="keywords=\{\"testone\",\"audio\"\},title=\"audio\ testtone\"" ! vorbisenc ! oggmux ! filesink location=test.ogg
  * ]| set keywords and title demonstrating quoting of special chars and handling lists
  * </refsect2>
  */
@@ -82,7 +82,7 @@ gst_tag_inject_finalize (GObject * object)
   GstTagInject *self = GST_TAG_INJECT (object);
 
   if (self->tags) {
-    gst_tag_list_free (self->tags);
+    gst_tag_list_unref (self->tags);
     self->tags = NULL;
   }
 
@@ -148,7 +148,7 @@ gst_tag_inject_transform_ip (GstBaseTransform * trans, GstBuffer * buf)
     if (self->tags && !gst_tag_list_is_empty (self->tags)) {
       GST_DEBUG ("tag event :%" GST_PTR_FORMAT, self->tags);
       gst_pad_push_event (GST_BASE_TRANSFORM_SRC_PAD (trans),
-          gst_event_new_tag ("GstTagInject", gst_tag_list_copy (self->tags)));
+          gst_event_new_tag (gst_tag_list_ref (self->tags)));
     }
   }
 
