@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -31,7 +31,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch souphttpsrc location=http://some.server/ iradio-mode=true ! icydemux ! fakesink -t
+ * gst-launch-1.0 souphttpsrc location=http://some.server/ iradio-mode=true ! icydemux ! fakesink -t
  * ]| This pipeline should read any available ICY tag information and output it.
  * The contents of the stream should be detected, and the appropriate mime
  * type set on buffers produced from icydemux. (Using gnomevfssrc, neonhttpsrc
@@ -131,7 +131,7 @@ gst_icydemux_reset (GstICYDemux * icydemux)
   gst_icydemux_remove_srcpad (icydemux);
 
   if (icydemux->cached_tags) {
-    gst_tag_list_free (icydemux->cached_tags);
+    gst_tag_list_unref (icydemux->cached_tags);
     icydemux->cached_tags = NULL;
   }
 
@@ -207,6 +207,28 @@ gst_icydemux_dispose (GObject * object)
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
+typedef struct
+{
+  GstCaps *caps;
+  GstPad *pad;
+} CopyStickyEventsData;
+
+static gboolean
+copy_sticky_events (GstPad * pad, GstEvent ** event, gpointer user_data)
+{
+  CopyStickyEventsData *data = user_data;
+
+  if (GST_EVENT_TYPE (*event) >= GST_EVENT_CAPS && data->caps) {
+    gst_pad_set_caps (data->pad, data->caps);
+    data->caps = NULL;
+  }
+
+  if (GST_EVENT_TYPE (*event) != GST_EVENT_CAPS)
+    gst_pad_push_event (data->pad, gst_event_ref (*event));
+
+  return TRUE;
+}
+
 static gboolean
 gst_icydemux_add_srcpad (GstICYDemux * icydemux, GstCaps * new_caps)
 {
@@ -225,6 +247,8 @@ gst_icydemux_add_srcpad (GstICYDemux * icydemux, GstCaps * new_caps)
   }
 
   if (icydemux->srcpad == NULL) {
+    CopyStickyEventsData data;
+
     icydemux->srcpad =
         gst_pad_new_from_template (gst_element_class_get_pad_template
         (GST_ELEMENT_GET_CLASS (icydemux), "src"), "src");
@@ -233,10 +257,12 @@ gst_icydemux_add_srcpad (GstICYDemux * icydemux, GstCaps * new_caps)
     gst_pad_use_fixed_caps (icydemux->srcpad);
     gst_pad_set_active (icydemux->srcpad, TRUE);
 
-    if (icydemux->src_caps) {
-      if (!gst_pad_set_caps (icydemux->srcpad, icydemux->src_caps))
-        GST_WARNING_OBJECT (icydemux, "Failed to set caps on src pad");
-    }
+    data.pad = icydemux->srcpad;
+    data.caps = icydemux->src_caps;
+    gst_pad_sticky_events_foreach (icydemux->sinkpad, copy_sticky_events,
+        &data);
+    if (data.caps)
+      gst_pad_set_caps (data.pad, data.caps);
 
     GST_DEBUG_OBJECT (icydemux, "Adding src pad with caps %" GST_PTR_FORMAT,
         icydemux->src_caps);
@@ -287,7 +313,7 @@ gst_icydemux_tag_found (GstICYDemux * icydemux, GstTagList * tags)
   } else {
     gst_tag_list_insert (icydemux->cached_tags, tags,
         GST_TAG_MERGE_REPLACE_ALL);
-    gst_tag_list_free (tags);
+    gst_tag_list_unref (tags);
   }
 
   return TRUE;
@@ -341,7 +367,7 @@ gst_icydemux_parse_and_send_tags (GstICYDemux * icydemux)
   if (!gst_tag_list_is_empty (tags))
     gst_icydemux_tag_found (icydemux, tags);
   else
-    gst_tag_list_free (tags);
+    gst_tag_list_unref (tags);
 }
 
 static gboolean
@@ -383,8 +409,12 @@ gst_icydemux_handle_event (GstPad * pad, GstObject * parent, GstEvent * event)
 
         return gst_pad_event_default (pad, parent, event);
       default:
-        icydemux->cached_events = g_list_append (icydemux->cached_events,
-            event);
+        if (!GST_EVENT_IS_STICKY (event)) {
+          icydemux->cached_events =
+              g_list_append (icydemux->cached_events, event);
+        } else {
+          gst_event_unref (event);
+        }
         return TRUE;
     }
   } else {
@@ -521,6 +551,7 @@ gst_icydemux_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
 
   if (icydemux->meta_interval == 0) {
     ret = gst_icydemux_typefind_or_forward (icydemux, buf);
+    buf = NULL;
     goto done;
   }
 
@@ -532,7 +563,12 @@ gst_icydemux_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
   while (size) {
     if (icydemux->remaining) {
       chunk = (size <= icydemux->remaining) ? size : icydemux->remaining;
-      sub = gst_buffer_copy_region (buf, GST_BUFFER_COPY_ALL, offset, chunk);
+      if (offset == 0 && chunk == size) {
+        sub = buf;
+        buf = NULL;
+      } else {
+        sub = gst_buffer_copy_region (buf, GST_BUFFER_COPY_ALL, offset, chunk);
+      }
       offset += chunk;
       icydemux->remaining -= chunk;
       size -= chunk;
@@ -576,7 +612,8 @@ gst_icydemux_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
   }
 
 done:
-  gst_buffer_unref (buf);
+  if (buf)
+    gst_buffer_unref (buf);
 
   return ret;
 
@@ -614,10 +651,7 @@ gst_icydemux_send_tag_event (GstICYDemux * icydemux, GstTagList * tags)
 {
   GstEvent *event;
 
-  gst_element_post_message (GST_ELEMENT (icydemux),
-      gst_message_new_tag (GST_OBJECT (icydemux), gst_tag_list_copy (tags)));
-
-  event = gst_event_new_tag ("GstDemuxer", tags);
+  event = gst_event_new_tag (tags);
   GST_EVENT_TIMESTAMP (event) = 0;
 
   GST_DEBUG_OBJECT (icydemux, "Sending tag event on src pad");
