@@ -134,6 +134,8 @@ gst_deinterleave_change_state (GstElement * element, GstStateChange transition);
 
 static gboolean gst_deinterleave_sink_event (GstPad * pad, GstObject * parent,
     GstEvent * event);
+static gboolean gst_deinterleave_sink_query (GstPad * pad, GstObject * parent,
+    GstQuery * query);
 
 static gboolean gst_deinterleave_src_query (GstPad * pad, GstObject * parent,
     GstQuery * query);
@@ -212,6 +214,8 @@ gst_deinterleave_init (GstDeinterleave * self)
       GST_DEBUG_FUNCPTR (gst_deinterleave_chain));
   gst_pad_set_event_function (self->sink,
       GST_DEBUG_FUNCPTR (gst_deinterleave_sink_event));
+  gst_pad_set_query_function (self->sink,
+      GST_DEBUG_FUNCPTR (gst_deinterleave_sink_query));
   gst_element_add_pad (GST_ELEMENT (self), self->sink);
 }
 
@@ -284,18 +288,22 @@ gst_deinterleave_add_new_pads (GstDeinterleave * self, GstCaps * caps)
   self->srcpads = g_list_reverse (self->srcpads);
 }
 
-static void
+static gboolean
 gst_deinterleave_set_pads_caps (GstDeinterleave * self, GstCaps * caps)
 {
   GList *l;
   gint i;
+  gboolean ret = TRUE;
 
   for (l = self->srcpads, i = 0; l; l = l->next, i++) {
     GstPad *pad = GST_PAD (l->data);
     GstCaps *srccaps;
     GstAudioInfo info;
 
-    gst_audio_info_from_caps (&info, caps);
+    if (!gst_audio_info_from_caps (&info, caps)) {
+      ret = FALSE;
+      continue;
+    }
     if (self->keep_positions)
       GST_AUDIO_INFO_POSITION (&info, 0) =
           GST_AUDIO_INFO_POSITION (&self->audio_info, i);
@@ -305,6 +313,7 @@ gst_deinterleave_set_pads_caps (GstDeinterleave * self, GstCaps * caps)
     gst_pad_set_caps (pad, srccaps);
     gst_caps_unref (srccaps);
   }
+  return ret;
 }
 
 static void
@@ -351,6 +360,56 @@ gst_deinterleave_set_process_function (GstDeinterleave * self)
   return TRUE;
 }
 
+static gboolean
+gst_deinterleave_check_caps_change (GstDeinterleave * self,
+    GstAudioInfo * old_info, GstAudioInfo * new_info)
+{
+  gint i;
+  gboolean same_layout = TRUE;
+  gboolean was_unpositioned;
+  gboolean is_unpositioned = GST_AUDIO_INFO_IS_UNPOSITIONED (new_info);
+  gint new_channels = GST_AUDIO_INFO_CHANNELS (new_info);
+  gint old_channels;
+
+  was_unpositioned = GST_AUDIO_INFO_IS_UNPOSITIONED (old_info);
+  old_channels = GST_AUDIO_INFO_CHANNELS (old_info);
+
+  /* We allow caps changes as long as the number of channels doesn't change
+   * and the channel positions stay the same. _getcaps() should've cared
+   * for this already but better be safe.
+   */
+  if (new_channels != old_channels)
+    goto cannot_change_caps;
+
+  /* Now check the channel positions. If we had no channel positions
+   * and get them or the other way around things have changed.
+   * If we had channel positions and get different ones things have
+   * changed too of course
+   */
+  if ((!was_unpositioned && is_unpositioned) || (was_unpositioned
+          && !is_unpositioned))
+    goto cannot_change_caps;
+
+  if (!is_unpositioned) {
+    if (GST_AUDIO_INFO_CHANNELS (old_info) !=
+        GST_AUDIO_INFO_CHANNELS (new_info))
+      goto cannot_change_caps;
+    for (i = 0; i < GST_AUDIO_INFO_CHANNELS (old_info); i++) {
+      if (new_info->position[i] != old_info->position[i]) {
+        same_layout = FALSE;
+        break;
+      }
+    }
+    if (!same_layout)
+      goto cannot_change_caps;
+  }
+
+  return TRUE;
+
+cannot_change_caps:
+  return FALSE;
+}
+
 static gboolean
 gst_deinterleave_sink_setcaps (GstDeinterleave * self, GstCaps * caps)
 {
@@ -366,50 +425,17 @@ gst_deinterleave_sink_setcaps (GstDeinterleave * self, GstCaps * caps)
     goto unsupported_caps;
 
   if (self->sinkcaps && !gst_caps_is_equal (caps, self->sinkcaps)) {
-    gint i;
-    gboolean same_layout = TRUE;
-    gboolean was_unpositioned;
-    gboolean is_unpositioned =
-        GST_AUDIO_INFO_IS_UNPOSITIONED (&self->audio_info);
-    gint new_channels = GST_AUDIO_INFO_CHANNELS (&self->audio_info);
-    gint old_channels;
     GstAudioInfo old_info;
 
     gst_audio_info_init (&old_info);
-    gst_audio_info_from_caps (&old_info, self->sinkcaps);
-    was_unpositioned = GST_AUDIO_INFO_IS_UNPOSITIONED (&old_info);
-    old_channels = GST_AUDIO_INFO_CHANNELS (&old_info);
+    if (!gst_audio_info_from_caps (&old_info, self->sinkcaps))
+      goto info_from_caps_failed;
 
-    /* We allow caps changes as long as the number of channels doesn't change
-     * and the channel positions stay the same. _getcaps() should've cared
-     * for this already but better be safe.
-     */
-    if (new_channels != old_channels ||
-        !gst_deinterleave_set_process_function (self))
-      goto cannot_change_caps;
-
-    /* Now check the channel positions. If we had no channel positions
-     * and get them or the other way around things have changed.
-     * If we had channel positions and get different ones things have
-     * changed too of course
-     */
-    if ((!was_unpositioned && is_unpositioned) || (was_unpositioned
-            && !is_unpositioned))
-      goto cannot_change_caps;
-
-    if (!is_unpositioned) {
-      if (GST_AUDIO_INFO_CHANNELS (&old_info) !=
-          GST_AUDIO_INFO_CHANNELS (&self->audio_info))
+    if (gst_deinterleave_check_caps_change (self, &old_info, &self->audio_info)) {
+      if (!gst_deinterleave_set_process_function (self))
         goto cannot_change_caps;
-      for (i = 0; i < GST_AUDIO_INFO_CHANNELS (&old_info); i++) {
-        if (self->audio_info.position[i] != old_info.position[i]) {
-          same_layout = FALSE;
-          break;
-        }
-      }
-      if (!same_layout)
-        goto cannot_change_caps;
-    }
+    } else
+      goto cannot_change_caps;
 
   }
 
@@ -424,7 +450,8 @@ gst_deinterleave_sink_setcaps (GstDeinterleave * self, GstCaps * caps)
   /* If we already have pads, update the caps otherwise
    * add new pads */
   if (self->srcpads) {
-    gst_deinterleave_set_pads_caps (self, srccaps);
+    if (!gst_deinterleave_set_pads_caps (self, srccaps))
+      goto set_caps_failed;
   } else {
     gst_deinterleave_add_new_pads (self, srccaps);
   }
@@ -450,6 +477,17 @@ invalid_caps:
     GST_ERROR_OBJECT (self, "invalid caps");
     return FALSE;
   }
+set_caps_failed:
+  {
+    GST_ERROR_OBJECT (self, "set_caps failed");
+    gst_caps_unref (srccaps);
+    return FALSE;
+  }
+info_from_caps_failed:
+  {
+    GST_ERROR_OBJECT (self, "coud not get info from caps");
+    return FALSE;
+  }
 }
 
 static void
@@ -482,15 +520,44 @@ __set_channels (GstCaps * caps, gint channels)
   }
 }
 
+static gboolean
+gst_deinterleave_sink_acceptcaps (GstPad * pad, GstObject * parent,
+    GstCaps * caps)
+{
+  GstDeinterleave *self = GST_DEINTERLEAVE (parent);
+  GstCaps *templ_caps = gst_pad_get_pad_template_caps (pad);
+  gboolean ret;
+
+  ret = gst_caps_can_intersect (templ_caps, caps);
+  gst_caps_unref (templ_caps);
+  if (ret && self->sinkcaps) {
+    GstAudioInfo new_info;
+
+    gst_audio_info_init (&new_info);
+    if (!gst_audio_info_from_caps (&new_info, caps))
+      goto info_from_caps_failed;
+    ret =
+        gst_deinterleave_check_caps_change (self, &self->audio_info, &new_info);
+  }
+
+  return ret;
+
+info_from_caps_failed:
+  {
+    GST_ERROR_OBJECT (self, "coud not get info from caps");
+    return FALSE;
+  }
+}
+
 static GstCaps *
-gst_deinterleave_sink_getcaps (GstPad * pad, GstObject * parent,
-    GstCaps * filter)
+gst_deinterleave_getcaps (GstPad * pad, GstObject * parent, GstCaps * filter)
 {
   GstDeinterleave *self = GST_DEINTERLEAVE (parent);
   GstCaps *ret;
-  GList *l;
+  GstIterator *it;
+  GstIteratorResult res;
+  GValue v = G_VALUE_INIT;
 
-  GST_OBJECT_LOCK (self);
   /* Intersect all of our pad template caps with the peer caps of the pad
    * to get all formats that are possible up- and downstream.
    *
@@ -499,50 +566,82 @@ gst_deinterleave_sink_getcaps (GstPad * pad, GstObject * parent,
    * will be detected here already
    */
   ret = gst_caps_new_any ();
-  for (l = GST_ELEMENT (self)->pads; l != NULL; l = l->next) {
-    GstPad *ourpad = GST_PAD (l->data);
-    GstCaps *peercaps = NULL, *ourcaps;
+  it = gst_element_iterate_pads (GST_ELEMENT_CAST (self));
 
-    ourcaps = gst_caps_copy (gst_pad_get_pad_template_caps (ourpad));
+  do {
+    res = gst_iterator_next (it, &v);
+    switch (res) {
+      case GST_ITERATOR_OK:{
+        GstPad *ourpad = GST_PAD (g_value_get_object (&v));
+        GstCaps *peercaps = NULL, *ourcaps;
+        GstCaps *templ_caps = gst_pad_get_pad_template_caps (ourpad);
 
-    if (pad == ourpad) {
-      if (GST_PAD_DIRECTION (pad) == GST_PAD_SINK)
-        __set_channels (ourcaps, GST_AUDIO_INFO_CHANNELS (&self->audio_info));
-      else
-        __set_channels (ourcaps, 1);
-    } else {
-      __remove_channels (ourcaps);
-      /* Only ask for peer caps for other pads than pad
-       * as otherwise gst_pad_peer_get_caps() might call
-       * back into this function and deadlock
-       */
-      peercaps = gst_pad_peer_query_caps (ourpad, NULL);
-      peercaps = gst_caps_make_writable (peercaps);
+        ourcaps = gst_caps_copy (templ_caps);
+        gst_caps_unref (templ_caps);
+
+        if (pad == ourpad) {
+          if (GST_PAD_DIRECTION (pad) == GST_PAD_SINK)
+            __set_channels (ourcaps,
+                GST_AUDIO_INFO_CHANNELS (&self->audio_info));
+          else
+            __set_channels (ourcaps, 1);
+        } else {
+          __remove_channels (ourcaps);
+          /* Only ask for peer caps for other pads than pad
+           * as otherwise gst_pad_peer_get_caps() might call
+           * back into this function and deadlock
+           */
+          peercaps = gst_pad_peer_query_caps (ourpad, NULL);
+          peercaps = gst_caps_make_writable (peercaps);
+        }
+
+        /* If the peer exists and has caps add them to the intersection,
+         * otherwise assume that the peer accepts everything */
+        if (peercaps) {
+          GstCaps *intersection;
+          GstCaps *oldret = ret;
+
+          __remove_channels (peercaps);
+
+          intersection = gst_caps_intersect (peercaps, ourcaps);
+
+          ret = gst_caps_intersect (ret, intersection);
+          gst_caps_unref (intersection);
+          gst_caps_unref (peercaps);
+          gst_caps_unref (oldret);
+        } else {
+          GstCaps *oldret = ret;
+
+          ret = gst_caps_intersect (ret, ourcaps);
+          gst_caps_unref (oldret);
+        }
+        gst_caps_unref (ourcaps);
+        g_value_reset (&v);
+        break;
+      }
+      case GST_ITERATOR_DONE:
+        break;
+      case GST_ITERATOR_ERROR:
+        gst_caps_unref (ret);
+        ret = gst_caps_new_empty ();
+        break;
+      case GST_ITERATOR_RESYNC:
+        gst_caps_unref (ret);
+        ret = gst_caps_new_empty ();
+        gst_iterator_resync (it);
+        break;
     }
+  } while (res != GST_ITERATOR_DONE && res != GST_ITERATOR_ERROR);
+  g_value_unset (&v);
+  gst_iterator_free (it);
 
-    /* If the peer exists and has caps add them to the intersection,
-     * otherwise assume that the peer accepts everything */
-    if (peercaps) {
-      GstCaps *intersection;
-      GstCaps *oldret = ret;
+  if (filter) {
+    GstCaps *aux;
 
-      __remove_channels (peercaps);
-
-      intersection = gst_caps_intersect (peercaps, ourcaps);
-
-      ret = gst_caps_intersect (ret, intersection);
-      gst_caps_unref (intersection);
-      gst_caps_unref (peercaps);
-      gst_caps_unref (oldret);
-    } else {
-      GstCaps *oldret = ret;
-
-      ret = gst_caps_intersect (ret, ourcaps);
-      gst_caps_unref (oldret);
-    }
-    gst_caps_unref (ourcaps);
+    aux = gst_caps_intersect (ret, filter);
+    gst_caps_unref (ret);
+    ret = aux;
   }
-  GST_OBJECT_UNLOCK (self);
 
   GST_DEBUG_OBJECT (pad, "Intersected caps to %" GST_PTR_FORMAT, ret);
 
@@ -594,6 +693,41 @@ gst_deinterleave_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
   return ret;
 }
 
+static gboolean
+gst_deinterleave_sink_query (GstPad * pad, GstObject * parent, GstQuery * query)
+{
+  gboolean res;
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_CAPS:{
+      GstCaps *filter;
+      GstCaps *caps;
+
+      gst_query_parse_caps (query, &filter);
+      caps = gst_deinterleave_getcaps (pad, parent, filter);
+      gst_query_set_caps_result (query, caps);
+      gst_caps_unref (caps);
+      res = TRUE;
+      break;
+    }
+    case GST_QUERY_ACCEPT_CAPS:{
+      GstCaps *caps;
+      gboolean ret;
+
+      gst_query_parse_accept_caps (query, &caps);
+      ret = gst_deinterleave_sink_acceptcaps (pad, parent, caps);
+      gst_query_set_accept_caps_result (query, ret);
+      res = TRUE;
+      break;
+    }
+    default:
+      res = gst_pad_query_default (pad, parent, query);
+      break;
+  }
+
+  return res;
+}
+
 static gboolean
 gst_deinterleave_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
 {
@@ -630,7 +764,7 @@ gst_deinterleave_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
     GstCaps *filter, *caps;
 
     gst_query_parse_caps (query, &filter);
-    caps = gst_deinterleave_sink_getcaps (pad, parent, filter);
+    caps = gst_deinterleave_getcaps (pad, parent, filter);
     gst_query_set_caps_result (query, caps);
     gst_caps_unref (caps);
   }
@@ -716,7 +850,8 @@ gst_deinterleave_process (GstDeinterleave * self, GstBuffer * buf)
      * here is an unliked pad */
     if (!buffers_out[i])
       goto alloc_buffer_failed;
-    else if (buffers_out[i] && gst_buffer_get_size (buffers_out[i]) != bufsize)
+    else if (buffers_out[i]
+        && gst_buffer_get_size (buffers_out[i]) != bufsize)
       goto alloc_buffer_bad_size;
 
     if (buffers_out[i]) {
