@@ -39,21 +39,21 @@
  * their output since they generally save metadata in the file header.
  * Therefore, it is often necessary that applications read the results in a bus
  * event handler for the tag message.  Obtaining the values this way is always
- * needed for <link linkend="GstRgAnalysis--num-tracks">album processing</link>
- * since the album gain and peak values need to be associated with all tracks of
- * an album, not just the last one.
+ * needed for album processing (see #GstRgAnalysis:num-tracks property) since
+ * the album gain and peak values need to be associated with all tracks of an
+ * album, not just the last one.
  * 
  * <refsect2>
  * <title>Example launch lines</title>
  * |[
- * gst-launch -t audiotestsrc wave=sine num-buffers=512 ! rganalysis ! fakesink
+ * gst-launch-1.0 -t audiotestsrc wave=sine num-buffers=512 ! rganalysis ! fakesink
  * ]| Analyze a simple test waveform
  * |[
- * gst-launch -t filesrc location=filename.ext ! decodebin \
+ * gst-launch-1.0 -t filesrc location=filename.ext ! decodebin \
  *     ! audioconvert ! audioresample ! rganalysis ! fakesink
  * ]| Analyze a given file
  * |[
- * gst-launch -t gnomevfssrc location=http://replaygain.hydrogenaudio.org/ref_pink.wav \
+ * gst-launch-1.0 -t gnomevfssrc location=http://replaygain.hydrogenaudio.org/ref_pink.wav \
  *     ! wavparse ! rganalysis ! fakesink
  * ]| Analyze the pink noise reference file
  * <para>
@@ -532,6 +532,7 @@ gst_rg_analysis_stop (GstBaseTransform * base)
   return TRUE;
 }
 
+/* FIXME: handle global vs. stream-tags? */
 static void
 gst_rg_analysis_handle_tags (GstRgAnalysis * filter,
     const GstTagList * tag_list)
@@ -632,13 +633,10 @@ gst_rg_analysis_handle_eos (GstRgAnalysis * filter)
       GST_LOG_OBJECT (filter, "posting tag list with results");
       gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
           GST_TAG_REFERENCE_LEVEL, filter->reference_level, NULL);
-      /* This steals our reference to the list: */
-      gst_pad_push_event (GST_BASE_TRANSFORM_SRC_PAD (GST_BASE_TRANSFORM
-              (filter)), gst_event_new_tag ("GstAnalysis",
-              gst_tag_list_copy (tag_list)));
-      /* special case element, also post here */
-      gst_element_post_message (GST_ELEMENT_CAST (filter),
-          gst_message_new_tag (GST_OBJECT (filter), tag_list));
+      /* This takes ownership of our reference to the list */
+      gst_pad_push_event (GST_BASE_TRANSFORM_SRC_PAD (filter),
+          gst_event_new_tag (tag_list));
+      tag_list = NULL;
     }
   }
 
@@ -657,6 +655,7 @@ gst_rg_analysis_handle_eos (GstRgAnalysis * filter)
     g_object_notify (G_OBJECT (filter), "num-tracks");
 }
 
+/* FIXME: return tag list (lists?) based on input tags.. */
 static gboolean
 gst_rg_analysis_track_result (GstRgAnalysis * filter, GstTagList ** tag_list)
 {
