@@ -33,14 +33,6 @@
 GST_DEBUG_CATEGORY_STATIC (v4l2vo_debug);
 #define GST_CAT_DEFAULT v4l2vo_debug
 
-/* Those are deprecated calls that have been replaced */
-#ifndef V4L2_CID_HCENTER
-#define V4L2_CID_HCENTER V4L2_CID_PAN_RESET
-#endif
-#ifndef V4L2_CID_VCENTER
-#define V4L2_CID_VCENTER V4L2_CID_TILT_RESET
-#endif
-
 void
 gst_v4l2_video_orientation_interface_init (GstVideoOrientationInterface * iface)
 {
@@ -64,18 +56,20 @@ gst_v4l2_video_orientation_get_vflip (GstV4l2Object * v4l2object,
   return gst_v4l2_get_attribute (v4l2object, V4L2_CID_VFLIP, flip);
 }
 
+/* named hcenter because of historical v4l2 naming */
 gboolean
 gst_v4l2_video_orientation_get_hcenter (GstV4l2Object * v4l2object,
     gint * center)
 {
-  return gst_v4l2_get_attribute (v4l2object, V4L2_CID_HCENTER, center);
+  return gst_v4l2_get_attribute (v4l2object, V4L2_CID_PAN_RESET, center);
 }
 
+/* named vcenter because of historical v4l2 naming */
 gboolean
 gst_v4l2_video_orientation_get_vcenter (GstV4l2Object * v4l2object,
     gint * center)
 {
-  return gst_v4l2_get_attribute (v4l2object, V4L2_CID_VCENTER, center);
+  return gst_v4l2_get_attribute (v4l2object, V4L2_CID_TILT_RESET, center);
 }
 
 gboolean
@@ -93,11 +87,11 @@ gst_v4l2_video_orientation_set_vflip (GstV4l2Object * v4l2object, gboolean flip)
 gboolean
 gst_v4l2_video_orientation_set_hcenter (GstV4l2Object * v4l2object, gint center)
 {
-  return gst_v4l2_set_attribute (v4l2object, V4L2_CID_HCENTER, center);
+  return gst_v4l2_set_attribute (v4l2object, V4L2_CID_PAN_RESET, center);
 }
 
 gboolean
 gst_v4l2_video_orientation_set_vcenter (GstV4l2Object * v4l2object, gint center)
 {
-  return gst_v4l2_set_attribute (v4l2object, V4L2_CID_VCENTER, center);
+  return gst_v4l2_set_attribute (v4l2object, V4L2_CID_TILT_RESET, center);
 }
