@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -53,7 +53,7 @@ check_bus_for_errors (GstBus * bus, GstClockTime max_wait_time)
     gst_message_parse_error (msg, &err, &debug);
     GST_ERROR ("ERROR: %s [%s]", err->message, debug);
     g_print ("\n===========> ERROR: %s\n%s\n\n", err->message, debug);
-    g_error_free (err);
+    g_clear_error (&err);
     g_free (debug);
     gst_message_unref (msg);
   }
@@ -127,6 +127,7 @@ test_with_caps (GstElement * src, GstElement * videocrop, GstCaps * caps)
   g_timer_destroy (timer);
   gst_object_unref (bus);
   gst_object_unref (pad);
+  gst_object_unref (pipeline);
 }
 
 /* return a list of caps where we only need to set
@@ -170,7 +171,7 @@ video_crop_get_test_caps (GstElement * videocrop)
 
 static gchar *opt_videosink_str;        /* NULL */
 static gchar *opt_filtercaps_str;       /* NULL */
-static gboolean opt_with_ffmpegcolorspace;      /* FALSE */
+static gboolean opt_with_videoconvert;  /* FALSE */
 
 int
 main (int argc, char **argv)
@@ -180,9 +181,9 @@ main (int argc, char **argv)
         "videosink to use (default: " DEFAULT_VIDEOSINK ")", NULL},
     {"caps", '\0', 0, G_OPTION_ARG_STRING, &opt_filtercaps_str,
         "filter caps to narrow down formats to test", NULL},
-    {"with-ffmpegcolorspace", '\0', 0, G_OPTION_ARG_NONE,
-          &opt_with_ffmpegcolorspace,
-          "whether to add an ffmpegcolorspace element in front of the sink",
+    {"with-videoconvert", '\0', 0, G_OPTION_ARG_NONE,
+          &opt_with_videoconvert,
+          "whether to add an videoconvert element in front of the sink",
         NULL},
     {NULL, '\0', 0, 0, NULL, NULL, NULL}
   };
@@ -200,8 +201,11 @@ main (int argc, char **argv)
 
   if (!g_option_context_parse (ctx, &argc, &argv, &opt_err)) {
     g_error ("Error parsing command line options: %s", opt_err->message);
+    g_option_context_free (ctx);
+    g_clear_error (&opt_err);
     return -1;
   }
+  g_option_context_free (ctx);
 
   GST_DEBUG_CATEGORY_INIT (videocrop_test_debug, "videocroptest", 0, "vctest");
 
@@ -217,9 +221,9 @@ main (int argc, char **argv)
   filter2 = gst_element_factory_make ("capsfilter", "capsfilter2");
   g_assert (filter2 != NULL);
 
-  if (opt_with_ffmpegcolorspace) {
-    g_print ("Adding ffmpegcolorspace\n");
-    csp = gst_element_factory_make ("ffmpegcolorspace", "colorspace");
+  if (opt_with_videoconvert) {
+    g_print ("Adding videoconvert\n");
+    csp = gst_element_factory_make ("videoconvert", "colorspace");
   } else {
     csp = gst_element_factory_make ("identity", "colorspace");
   }
@@ -276,10 +280,10 @@ main (int argc, char **argv)
     g_error ("Failed to link videoscale to capsfilter2");
 
   if (!gst_element_link (filter2, csp))
-    g_error ("Failed to link capsfilter2 to ffmpegcolorspace");
+    g_error ("Failed to link capsfilter2 to videoconvert");
 
   if (!gst_element_link (csp, sink))
-    g_error ("Failed to link ffmpegcolorspace to video sink");
+    g_error ("Failed to link videoconvert to video sink");
 
   caps_list = video_crop_get_test_caps (crop);
   for (l = caps_list; l != NULL; l = l->next) {
